/*
 * Thumb-1 functions.
 *
 * Copyright 2016 Ling LI <lix2ng@gmail.com>, under the MIT license.
 */

#ifndef THUMB1_H_
#define THUMB1_H_

/* Count leading zeros, any input. */
extern int thumb1_clz(uint32_t x);

/* Count leading zeros for known non-zero input.
 * Same as thumb1_clz() in this implementation, just for symmetry. */
extern int thumb1_clznz(uint32_t x);

/* Count trailing zeros, any input. */
extern int thumb1_ctz(uint32_t x);

/* Count trailing zeros for known non-zero input. */
extern int thumb1_ctznz(uint32_t x);

/* Reverse all bits. */
extern uint32_t thumb1_rbit(uint32_t x);

/* High 32 bits of unsigned 32x32->64 product (UMULL) */
extern uint32_t thumb1_umulh(uint32_t a, uint32_t b);

/* High 32 bits of signed 32x32->64 product (SMULL). */
extern int32_t thumb1_smulh(int32_t a, int32_t b);

/*
 * Signed and unsigned integer division 32/16->16.
 * - quotient is 16-bit, dividend therefore must be a product of two 16-bit
 *   numbers, or overflow would ensue.
 * - division by 0, including 0/0, and overflow all return +/-65535.
 * - all numbers are in native registers, there is no size conversion (UXTH,
 *   SXTH).
 */
extern int32_t thumb1_sdiv16(int32_t a, int32_t b);
extern uint32_t thumb1_udiv16(uint32_t a, uint32_t b);

#endif /* THUMB1_H_ */

/* vim: set syn=c.doxygen cin et sw=4 ts=4 tw=80 fo=croqm: */
